/*
		      ANSI C Runtime Library

    Copyright 1983,1984,1985,1986,1987,1988,1989,1990,1991,1992,1993,1994,1995
		    Green Hills Software, Inc.

    This program is the property of Green Hills Software, Inc,
    its contents are proprietary information and no part of it
    is to be disclosed to anyone except employees of Green Hills
    Software, Inc., or as agreed in writing signed by the President
    of Green Hills Software, Inc.
*/

#ifndef _MATH_H
#define _MATH_H

#ifdef __cplusplus
extern "C" {
#endif

#define EDOM    33	/* also #define'd in errno.h */
#define ERANGE  34	/* also #define'd in errno.h */

#ifndef __NoFloat
#if defined(__IeeeFloat)	/* from float.h */
# define HUGE_VAL       (1.7976931348623158e+308)  /* Max IEEE double exactly */
# ifndef _POSIX_SOURCE
#  define HUGE		HUGE_VAL
#  define HUGEFLOAT	(3.4028235e+38)	/* Max IEEE single exactly */
# endif
#elif defined(__VaxFloat)
# define HUGE_VAL         1e+37
# ifndef _POSIX_SOURCE
#  define HUGE		HUGE_VAL
#  define HUGEFLOAT	HUGE_VAL
# endif
#else
# error   Neither IEEE or Vax Floating point defined.  This is very bad.
#endif

#if defined(__STDC__) || defined(__PROTOTYPES__)
# define __ARGS(x)	x
#else
# define __ARGS(x)	()
  double cabs();
#endif

double atof __ARGS((const char *));

double acos	__ARGS((double));
double asin	__ARGS((double));
double atan	__ARGS((double));
double atan2	__ARGS((double, double));
double cos	__ARGS((double));
double sin	__ARGS((double));
double tan	__ARGS((double));

double cosh	__ARGS((double));
double sinh	__ARGS((double));
double tanh	__ARGS((double));

double exp	__ARGS((double));
double frexp	__ARGS((double, int *));
double ldexp	__ARGS((double, int));
double log	__ARGS((double));
double log10	__ARGS((double));
double modf	__ARGS((double, double *));

double pow	__ARGS((double, double));
double sqrt	__ARGS((double));

double ceil	__ARGS((double));
double fabs	__ARGS((double));
double floor	__ARGS((double));
double fmod	__ARGS((double, double));

#if defined(__cplusplus) || (__STDC__==0) && !defined(_POSIX_SOURCE)
double acosh	__ARGS((double));	/* neither Ansi nor Unix has these */
double asinh	__ARGS((double));	/* neither Ansi nor Unix has these */
double atanh	__ARGS((double));	/* neither Ansi nor Unix has these */
double erf	__ARGS((double));
double erfc	__ARGS((double));
double j0	__ARGS((double));
double j1	__ARGS((double));
double jn	__ARGS((int, double));
double y0	__ARGS((double));
double y1	__ARGS((double));
double yn	__ARGS((int, double));
double gamma	__ARGS((double));
double hypot	__ARGS((double, double));
int isnan	__ARGS((double));
int isinf	__ARGS((double));
#endif	/* _MATH_EXTENSIONS */

/* Ansi said these might be added to the standard later, but we have them now */
#ifndef _POSIX_SOURCE
float acosf	__ARGS((float));
float asinf	__ARGS((float));
float atanf	__ARGS((float));
float atan2f	__ARGS((float, float));
float cosf	__ARGS((float));
float sinf	__ARGS((float));
float tanf	__ARGS((float));
float coshf	__ARGS((float));
float sinhf	__ARGS((float));
float tanhf	__ARGS((float));
float expf	__ARGS((float));
float frexpf	__ARGS((float, int *));
float ldexpf	__ARGS((float, int));
float logf	__ARGS((float));
float log10f	__ARGS((float));
float sqrtf	__ARGS((float));
float powf	__ARGS((float, float));
#endif 		/* _POSIX_SOURCE */

#if __STDC__==0 && !defined(_POSIX_SOURCE)
#define M_PI	   3.14159265358979323846
#define M_PI_2	   1.570796326794896619231
#define M_PI_4	   0.785398163397448309616
#define M_1_PI	   0.318309886183790671538
#define M_2_PI	   0.636619772367581343076
#define M_E	   2.718281828459045235360
#define M_LN2	   0.69314718055994530942
#define M_LN10	   2.30258509299404568402
#define M_SQRT2	   1.41421356237309504880
#define M_LOG10E   0.434294481903251827651
#define M_LOG2E	   1.442695040888963407
#define M_2_SQRTPI 1.12837916709551257390
#endif	/* __NoFloat */

#define DOMAIN		1
#define	SING		2
#define	OVERFLOW	3
#define	UNDERFLOW	4
#define	TLOSS		5
#define	PLOSS		6

#ifndef __NoFloat
struct exception {
    int type;
    char *name;
    double arg1;
    double arg2;
    double retval;
};
#ifndef __REXCEPTION__S
#define __REXCEPTION__S
struct rexception {
    int type;
    char *name;
    float arg1;
    float arg2;
    float retval;
};
#endif  /* __REXCEPTION__S */
#endif	/* __STDC__ == 0 and !_POSIX_SOURCE */
#endif	/* __NoFloat */

#ifdef __cplusplus
	}
#endif
#undef __ARGS
#endif	/* _MATH_H */
