/***********************************************************************
<t@C>	sound.h

	Copyright (c) 1995-1997 Sharp Corporation.
	All rights reserved.

	Ver.1.0	for MI-10
***********************************************************************/


#ifndef _SOUND_H
#define _SOUND_H

#include        <drfio.h>

/* import from rpeltpsb.h */

#define            SEGMENT_SIZE            160

/*************************/
/*	Type Define 	 */
/*************************/
typedef		unsigned char	uchar;
typedef		unsigned long	ulong;
#ifndef __sys_types_h
typedef		unsigned short	ushort;
#endif

#include	<sfconv.h>

/* DEFINE STRUCTS */

typedef	struct {
	short		nc;
	short		bc;
	short		mc;
	short		xmaxc;
	short		xmc[13];
} SUB_SEG_DT;

typedef	struct	{
	long		L_z2;  
	short		Np;
	short		Mp;
	short		Z1;
	short		Msr;
	short		Exp;
	short		Mant;
	short		Wt[50];	
	short		Dp[ 120 ];
	short		S_data[SEGMENT_SIZE];
	short		S_para[ 18 ];
	short		Larc[ 9 ];
	short		Larpp[ 9 ];
	short		U[8];
	short		V[ 10 ];
	short		encFlag;		/* for encoder initial	*/
	short		decFlag;		/* for decoder initial	*/
	long		unit;			/* PCM process size	*/
	SUB_SEG_DT 	Ssc[ 4 ];
} RpeWork;


/* import from conv.h */

#define		END_OF_DATA	(-1)
#define		DATA_BUFF_FULL	(-2)
#define		UNKNOWN_SIZE	(~((ulong)0))	/* for .au file	*/
#define		INIT_VAL	(~((ulong)0))
#define		ZSD_SIZE_ALL	(~((ulong)0))

/* fileID VALUES */
#define         File_ZSD        0
#define         File_AU         1
#define         File_WAV        2

/* compID VALUES */
#define         Comp_PCM16      0
#define         Comp_PCM8       1
#define         Comp_ULAW       2
#define         Comp_RPE        3

#define		ZauCompIDs	4
#define		AuCompIDs	(ZauCompIDs + 3)
#define		WavCompIDs	(AuCompIDs + 2)

#define		GET_MODULE_NUMBER(fid,cid) \
	( ((fid)==File_WAV) ? (cid)+AuCompIDs : \
		( ((fid)==File_AU) ? (cid)+ZauCompIDs : (cid) ) )
#define		PUT_MODULE_NUMBER(fid,cid) \
	( ((fid)==File_WAV) ? (cid)+AuCompIDs : \
		( ((fid)==File_AU) ? (cid)+ZauCompIDs : (cid) ) )

/* for flconv */
#define	SND_PROC_BLOCK	320
#define	RPE_UNIT_CODE	65
#define	MAX_UP_RATE	6
#define	Bsize		(SND_PROC_BLOCK * MAX_UP_RATE)

#define GET_CODE_SIZE( ID, pcm_size, code_size )  \
  switch( ID ){ \
  case Comp_PCM8: \
  case Comp_ULAW: (code_size)=(pcm_size); break; \
  case Comp_RPE: (code_size)=((pcm_size)/SND_PROC_BLOCK)*RPE_UNIT_CODE; break; \
  case Comp_PCM16: \
  default: (code_size)=(pcm_size)*2; break; \
  }

#define GET_PCM_SIZE( ID, code_size, pcm_size )  \
  switch( ID ){ \
  case Comp_PCM8: \
  case Comp_ULAW: (pcm_size)=(code_size); break; \
  case Comp_RPE: (pcm_size)=((code_size)/RPE_UNIT_CODE)*SND_PROC_BLOCK; break; \
  case Comp_PCM16: \
  default: (pcm_size)=(code_size)/2; break; \
  }

#define		CONDITION_COMMAND	 0
#define		CONDITION_SIZE		 1
#define		CONDITION_RECSIZE	 2
#define		CONDITION_ERROR		 3
#define		CONDITION_LEVEL		 4
#define		CONDITION_DATASIZE	 5
#define		CONDITION_STATUS	 6
#define		CONDITION_ALERTNUM	 7

#define		STATUS_NULL		0x00
#define		STATUS_REC		0x01
#define		STATUS_PLAY		0x02
#define		STATUS_ALERT		0x04
#define		STATUS_PAUSE		0x08
#define		STATUS_END		0x10
#define		STATUS_TRIGGER		0x20	/**/
#define		STATUS_EFFECT		0x40	/**/

#define		FLAG_UP			 1
#define		FLAG_DOWN		(-1)

#define		CMND_NULL		 0
#define		CMND_PAUSE		 1
#define		CMND_STOP		 2

#define		NORMAL_END		0
#define		ERR_fileID		(-1)
#define		ERR_compID		(-2)
#define		ERR_SmplFreq		(-3)
#define		ERR_MemCreate		(-4)
#define		ERR_Condition		(-5)
#define		ERR_System		(-6)
#define		ERR_BuffFull		(-7)
#define		ERR_AlertNum		(-8)
#define		ERR_ProcFork		(-9)
#define		ERR_Version		(-10)
#define		ERR_Format		(-11)
#define		ERR_Channel		(-12)
#define		ERR_Sfconv		(-13)
#define		ERR_FileRead		(-14)
#define		ERR_FileWrite		(-15)
#define		ERR_TimeOut		(-16)
#define		ERR_Mode		(-17)
#define		ERR_Malloc		(-18)
#define		ERR_Free		(-19)
#define		ERR_FileOpen		(-20)
#define		ERR_FileType		(-21)

#define		SNDPROC_PRIORITY	1
#define		ALERT_MAX_NUM		3
#define		SNDPROC_NOPROC		(-1)

/* input mode */
#define		SNDPROC_FILEP		0
#define		SNDPROC_FNAME		1
/* release mode */
#define		SNDPROC_WAIT		0x00
#define		SNDPROC_RELEASE		0x10
#define		SNDPROC_EFFECT		0x20
/* endFlag */
#define		SNDPROC_RUNNING		0
#define		SNDPROC_END		1

/*********************************/
/*	Struct Type Define 	 */
/*********************************/
typedef	struct	{
	ushort	fileID,
		compID;
	ulong	smplFreq;
	ushort	channelNum;
	ulong	hdrSize;
	ulong	dataSize;
	ulong	unitDataSize;
} HDR;

typedef	struct	{
	char	fileID[4];
	uchar	Version;
	uchar	Channel;
	ushort	CodeMeth;
	ulong	Sampling;
	ulong	DataSize;
	ulong	DataOffs;
} ZauHead;

typedef	struct	{
	char	fileID[4];
	ulong	HeadSize,
		DataSize,
		DataForm,
		Sampling,
		Channel;
} AuHead;

typedef	struct	{
	char	fileID[4];
	ulong	RiffSize;
	char	WaveID[4],
		FmtChunkID[4];
	ulong	FmtSize;
	ushort	FormTag,
		Channel;
	ulong	Sampling,
		AvgByte;
	ushort	Block,
		BitsSampl;
	char	DatChunkID[4];
	ulong	DataSize;
} WavHead;

typedef	struct	{
	DRFFILE	*rPtr, *wPtr;
	HDR	in_hdr, out_hdr;
	int	getFunc, putFunc;
	ushort	pcmIn, codeIn;
	ushort	pcmOut, codeOut;
	short	shift;
	short	*pcmPtr;
	long	read;
	ulong	totalRead;
	CNV_PARA cnv;		/* parameters for sfconv        */
	short	buff1[Bsize], buff2[Bsize], wave[Bsize];
	RpeWork rpe;
} Fcnv;

typedef	struct	{
	short	error;
	char	smplFreq;
	char	speaker;
	char	intMask;	
} SaveDat;

typedef struct {
	HDR	*zsd;		/* recording condition struct */
	DRFFILE *ptr;		/* {Data,Resource,File} pointer */
	ulong	size;		/* current size (with header) */
	ulong	maxSize;	/* maximum size (with header) */
	int	endFlag;	/* set 1 when process terminated */
	int	volume;		/* alert only */
	int	type;		/* SNDPROC_{FILEP,FNAME}|SNDPROC_RELEASE} */
	char	*fname;		/* filename */
	int	priority;	/* sound process priority */
} SndParam;
#define RecParam	SndParam
#define PlayParam	SndParam
#define AlertParam	SndParam

/* convmng.c	*/
int	ZsdSetCond(ushort fileID, ushort compID, ulong smplFreq, HDR *zsd);
int	ZsdRecGo(RecParam *param);
int	ZsdGetCond(DRFFILE *zfp, HDR *zsd);
int	ZsdPlayGo(PlayParam *param);
int	ZsdAlert(AlertParam *param);
int	ZsdGetStatus();
int	ZsdSetStatus(int mask, int updown, int *old);
int	ZsdGetCondition(int No);
int	ZsdSetCondition(int No, int Value );
int	ZsdGetStruct(HDR *zsd);
int	ZsdSetStruct(HDR *zsd);
int	ZsdGetTime(HDR *zsd);
ulong	ZsdSize2Time(HDR *zsd, ulong size);
ulong	ZsdTime2Size(HDR *zsd, ulong time);

int	ZsdAlertDRF(DRFFILE *drf, int volume, int priority);
int	ZsdAlertFname(char *fname, int volume, int priority);
int	ZsdEffectDRF(DRFFILE *drf, int volume, int priority);
int	ZsdEffectFname(char *fname, int volume, int priority);
void	*ZsdCreateSndParam();
int	ZsdDisposeSndParam(void *param);
int	ZsdRecDRF(RecParam *param, DRFFILE *drf, int maxSize);
int	ZsdPlayDRF(RecParam *param, DRFFILE *drf, int start, int maxSize);
int	ZsdPlayFname(RecParam *param, char *fname, int start, int maxSize);
int	ZsdPause(void *param);
int	ZsdCont(void *param);
int	ZsdStop(void *param);
DRFFILE *ZsdDRFmalloc(ulong size);
int	ZsdDRFfree(DRFFILE *drf);
DRFFILE *ZsdDRFfopen(char *fname, char *mode);
int	ZsdDRFfclose(DRFFILE *drf);

/* snddrive.c	*/
int	AD_INIT();
int	AD_START();
int	AD_END();
int	AD_MIC(short value);
int	AD_GAIN(char value);
int	AGC_MODE(short value);
int	DA_INIT();
int	DA_START();
int	DA_STOP();
int	DA_FLUSH();
int	DA_END();
int	DA_GAIN(char value);
int	DA_SPEAKER(short value);
int	AD_DA_InitSet();
ulong	Set_Freq(ulong freq);
int	AD_DA_PtrSet(short bias);
int	GET_SysMemPtr(short *getPtr[]);

#endif	/* _SOUND_H	*/

